//
// (c) 2006 DS Data Systems UK Ltd, All rights reserved.
//
// DS Data Systems and KonaKart and their respective logos, are 
// trademarks of DS Data Systems UK Ltd. All rights reserved.
//
// The information in this document is free software; you can redistribute 
// it and/or modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// Original version written by Steven Lohrenz (steven@stevenlohrenz.com) 
// based on a KonaKart example.  
//

package com.konakart.bl.modules.payment.worldpay;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import com.konakart.app.KKConfiguration;
import com.konakart.app.KKException;
import com.konakart.app.NameValue;
import com.konakart.app.Order;
import com.konakart.app.OrderTotal;
import com.konakart.app.PaymentDetails;
import com.konakart.appif.CountryIf;
import com.konakart.appif.KKEngIf;
import com.konakart.bl.modules.BaseModule;
import com.konakart.bl.modules.ordertotal.OrderTotalMgr;
import com.konakart.bl.modules.payment.BasePaymentModule;
import com.konakart.bl.modules.payment.PaymentInfo;
import com.konakart.bl.modules.payment.PaymentInterface;

/**
 * WorldPay IPN module
 * 
 */
public class Worldpay extends BasePaymentModule implements PaymentInterface
{

    // Module name must be the same as the class name although it can be all in lowercase in order
    // to remain compatible with osCommerce.
    private static String code = "worldpay";

    private static int sortOrder = -1;

    // The WorldPay Url used to POST the payment request.
    // "https://secure.worldpay.com/index_shop.cgi"
    private static String worldPayRequestUrl;

    private static String worldPayInstId;

    private static String worldPayTestMode;

    private static int zone;

    private static String bundleName = BaseModule.basePackage + ".payment.worldpay.Worldpay";

    private static HashMap<Locale, ResourceBundle> resourceBundleMap = new HashMap<Locale, ResourceBundle>();

    private static String mutex = "worldpayMutex";

    // Configuration Keys

    /**
     * Used to put the gateway online / offline
     */
    private final static String MODULE_PAYMENT_WORLDPAY_STATUS = "MODULE_PAYMENT_WORLDPAY_STATUS";

    /**
     * The WorldPay zone, if greater than zero, should reference a GeoZone. If the DeliveryAddress
     * of the order isn't within that GeoZone, then we throw an exception
     */
    private final static String MODULE_PAYMENT_WORLDPAY_ZONE = "MODULE_PAYMENT_WORLDPAY_ZONE";

    /**
     * The order for displaying this payment gateway on the UI
     */
    private final static String MODULE_PAYMENT_WORLDPAY_SORT_ORDER = "MODULE_PAYMENT_WORLDPAY_SORT_ORDER";

    /**
     * The WorldPay Url used to POST the payment request.
     * "https://secure.worldpay.com/index_shop.cgi"
     */
    private final static String MODULE_PAYMENT_WORLDPAY_REQUEST_URL = "MODULE_PAYMENT_WORLDPAY_REQUEST_URL";

    /**
     * The WorldPay ID for this installation
     */
    private final static String MODULE_PAYMENT_WORLDPAY_INST_ID = "MODULE_PAYMENT_WORLDPAY_INST_ID";

    /**
     * 100 is testing that always returns successful transaction. 101 is testing that always returns
     * an unsuccessful transaction. 0 is a live site.
     * 
     */
    private final static String MODULE_PAYMENT_WORLDPAY_TEST_MODE = "MODULE_PAYMENT_WORLDPAY_TEST_MODE";

    // Message Catalogue Keys
    private final static String MODULE_PAYMENT_WORLDPAY_TEXT_TITLE = "module.payment.worldpay.text.title";

    private final static String MODULE_PAYMENT_WORLDPAY_TEXT_DESCRIPTION = "module.payment.worldpay.text.description";

    /**
     * Constructor
     * 
     * @param eng
     * 
     * @throws KKException
     */
    public Worldpay(KKEngIf eng) throws KKException
    {
        super.init(eng);

        // Initialise the static variables
        if (sortOrder == -1)
        {
            synchronized (mutex)
            {
                if (sortOrder == -1)
                {
                    setStaticVariables();
                }
            }
        }
    }

    /**
     * Sets some static variables during setup
     * 
     * @throws KKException
     * 
     */
    public void setStaticVariables() throws KKException
    {
        KKConfiguration conf;

        conf = getEng().getConfiguration(MODULE_PAYMENT_WORLDPAY_REQUEST_URL);
        if (conf == null)
        {
            throw new KKException(
                    "The Configuration MODULE_PAYMENT_WORLDPAY_REQUEST_URL must be set to the URL for"
                            + " sending the request to WorldPay. (i.e. https://select.worldpay.com/wcc/purchase)");
        }
        worldPayRequestUrl = conf.getValue();

        conf = getEng().getConfiguration(MODULE_PAYMENT_WORLDPAY_INST_ID);
        if (conf == null)
        {
            throw new KKException(
                    "The Configuration MODULE_PAYMENT_WORLDPAY_INST_ID must be set to the"
                            + " WorldPay ID for this installation");
        }
        worldPayInstId = conf.getValue();

        conf = getEng().getConfiguration(MODULE_PAYMENT_WORLDPAY_TEST_MODE);
        if (conf == null)
        {
            throw new KKException(
                    "The Configuration MODULE_PAYMENT_WORLDPAY_TEST_MODE must be set to on eof the following values: "
                            + "100 is testing that always returns successful transaction. "
                            + "101 is testing that always returns an unsuccessful transaction. "
                            + "0 is a live site.");
        }
        worldPayTestMode = conf.getValue();

        conf = getEng().getConfiguration(MODULE_PAYMENT_WORLDPAY_ZONE);
        if (conf == null)
        {
            zone = 0;
        } else
        {
            zone = new Integer(conf.getValue()).intValue();
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_WORLDPAY_SORT_ORDER);
        if (conf == null)
        {
            sortOrder = 0;
        } else
        {
            sortOrder = new Integer(conf.getValue()).intValue();
        }
    }

    /**
     * Return a payment details object for WorldPay IPN module
     * 
     * @param order
     * @param info
     * @return Returns information in a PaymentDetails object
     * @throws Exception
     */
    public PaymentDetails getPaymentDetails(Order order, PaymentInfo info) throws Exception
    {

        /*
         * The WorldPayZone zone, if greater than zero, should reference a GeoZone. If the
         * DeliveryAddress of the order isn't within that GeoZone, then we throw an exception
         */
        if (zone > 0)
        {
            checkZone(info, zone);
        }

        // Get the scale for currency calculations
        int scale = new Integer(order.getCurrency().getDecimalPlaces()).intValue();

        // Get the resource bundle
        ResourceBundle rb = getResourceBundle(mutex, bundleName, resourceBundleMap, info
                .getLocale());
        if (rb == null)
        {
            throw new KKException("A resource file cannot be found for the country "
                    + info.getLocale().getCountry());
        }

        PaymentDetails pDetails = new PaymentDetails();
        pDetails.setCode(code);
        pDetails.setSortOrder(sortOrder);
        pDetails.setPaymentType(PaymentDetails.BROWSER_PAYMENT_GATEWAY);
        pDetails.setDescription(rb.getString(MODULE_PAYMENT_WORLDPAY_TEXT_DESCRIPTION));
        pDetails.setTitle(rb.getString(MODULE_PAYMENT_WORLDPAY_TEXT_TITLE));

        // Return now if the full payment details aren't required. This happens when the manager
        // just wants a list of payment gateways to display in the UI.
        if (info.isReturnDetails() == false)
        {
            return pDetails;
        }

        pDetails.setPostOrGet("post");
        pDetails.setRequestUrl(worldPayRequestUrl);

        List<NameValue> parmList = new ArrayList<NameValue>();
        /*
         * Obligatory parameters
         */
        parmList.add(new NameValue("instId", worldPayInstId));
        parmList.add(new NameValue("currency", order.getCurrency().getCode()));
        parmList.add(new NameValue("cartId", order.getId()));
        parmList.add(new NameValue("desc", "Order #" + order.getId() + " from "
                + info.getStoreName()));

        // Worldpay only requires details of the final price. No tax, subtotal etc.
        BigDecimal total = null;
        for (int i = 0; i < order.getOrderTotals().length; i++)
        {
            OrderTotal ot = (OrderTotal) order.getOrderTotals()[i];
            if (ot.getClassName().equals(OrderTotalMgr.ot_total))
            {
                total = ot.getValue().setScale(scale, BigDecimal.ROUND_HALF_UP);
            }
        }

        if (total == null)
        {
            throw new KKException("An Order Total was not found");
        }

        parmList.add(new NameValue("amount", total.toString()));

        /*
         * Optional parameters
         */
        parmList.add(new NameValue("testMode", worldPayTestMode));

        // Set one of the custom variables with the secret key
        parmList.add(new NameValue("M_cs1", info.getSecretKey()));
        parmList.add(new NameValue("M_cs2", order.getId()));

        // Set the billing address
        parmList.add(new NameValue("name", order.getBillingName()));
        parmList.add(new NameValue("postcode", order.getBillingPostcode()));
        parmList.add(new NameValue("tel", order.getCustomerTelephone()));
        parmList.add(new NameValue("email", order.getCustomerEmail()));

        StringBuffer addrSB = new StringBuffer();
        addrSB.append(order.getBillingStreetAddress());
        addrSB.append("&#10;");
        if (order.getBillingSuburb() != null && order.getBillingSuburb().length() > 0)
        {
            addrSB.append(order.getBillingSuburb());
            addrSB.append("&#10;");
        }
        addrSB.append(order.getBillingCity());
        addrSB.append("&#10;");
        if (order.getBillingState() != null && order.getBillingState().length() > 0)
        {
            addrSB.append(order.getBillingState());
            addrSB.append("&#10;");
        }
        parmList.add(new NameValue("address", addrSB.toString()));

        // Country requires the two letter country code
        CountryIf country = getEng().getCountryPerName(order.getBillingCountry());
        if (country != null)
        {
            parmList.add(new NameValue("country", country.getIsoCode2()));
        }

        // Put the parameters into an array
        NameValue[] nvArray = new NameValue[parmList.size()];
        parmList.toArray(nvArray);
        pDetails.setParameters(nvArray);

        if (log.isDebugEnabled())
        {
            log.debug(pDetails.toString());
        }

        return pDetails;
    }

    /**
     * Returns true or false
     * 
     * @throws KKException
     */
    public boolean isAvailable() throws KKException
    {
        return isAvailable(getEng(), MODULE_PAYMENT_WORLDPAY_STATUS);
    }

}
